/*
 *	Copyright 2011 CiviGuard, Inc.
 *	Copyright 2011 Mihai Tomescu <matomesc@gmail.com>
 *
 *	Licensed under the Apache License, Version 2.0 (the "License");
 *	you may not use this file except in compliance with the License.
 *	You may obtain a copy of the License at
 *
 *	    http://www.apache.org/licenses/LICENSE-2.0
 *
 *	Unless required by applicable law or agreed to in writing, software
 *	distributed under the License is distributed on an "AS IS" BASIS, 
 *	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *	See the License for the specific language governing permissions and
 *	limitations under the License.
 */

var EventEmitter = require('eventemitter2').EventEmitter2;
	Memcached = require('memcached'),
	util = require('util');

exports.version = '0.0.1';

var Client = exports.Client = function(server, queue) {
	EventEmitter.call(this);
	var self = this;
	self.queue = queue;
	self._memcached = new Memcached(server);
	
	var hit_queue = function() {
		self.open('500', function(err, result) {
			if (!err) {
				if (result) self.emit('item', result);
				else self.emit('miss');	
			} else {
				self.emit('error.open');
			}
		});
	}
	self.on('miss', function(event) {
		hit_queue();
	});
	self.on('close', function(event) {
		hit_queue();
	});
	self.on('abort', function(event) {
		hit_queue();
	});
	hit_queue();
};

util.inherits(Client, EventEmitter);

Client.prototype.add = function(value, lifetime, callback) {
	var self = this;
	var lt = (typeof lifetime == 'function') ? 0 : lifetime;
	var callback = (typeof lifetime == 'function') ? lifetime : callback;
	self._memcached.set(self.queue, value, lt, callback);
};

Client.prototype.open = function(timeout, callback) {
	var opts = '';
	if (typeof timeout == 'function') {
		callback = timeout;
	} else {
		opts += '/t=' + timeout;
	}
	opts += '/open';
	this.get(opts, callback);
}

Client.prototype.close = function() {
	var self = this;
	var opts = '/close';
	this.get(opts, function(err, result) {
		if (!err) {
			self.emit('close');
		} else {
			self.emit('error.close');
		}
	});
}

Client.prototype.abort = function(callback) {
	var self = this;
	var opts = '/abort';
	this.get(opts, function(err, result) {
		if (!err) {
			self.emit('abort');
		} else {
			self.emit('error.abort');
		}
	});
}

Client.prototype.peek = function(timeout, callback) {
	var opts = '';
	if (typeof timeout == 'function') {
		callback = timeout;
	} else {
		opts += '/t=' + timeout;
	}
	opts += '/peek';
	this.get(opts, callback);
}

Client.prototype.get = function(opts, callback) {
	var self = this;
	var cmd = self.queue;
	if (typeof opts == 'function') {
		callback = opts;
	} else {
		cmd += opts;
	}
	self._memcached.get(cmd, callback);
};

Client.prototype.delete = function(callback) {
	var self = this;
	self._memcached.delete(self.queue, callback);
};

Client.prototype.flush = function(callback) {
	var self = this;
	self._memcached.query('FLUSH ' + self.queue, 'simple', callback);
}

Client.prototype.flush_all = function(callback) {
	var self = this;
	self._memcached.query('FLUSH_ALL', 'simple', callback);
};

Client.prototype.version = function(callback) {
	var self = this;
	self._memcached.version(callback);
}

Client.prototype.shutdown = function(callback) {
	// TODO
}

Client.prototype.reload = function(callback) {
	// TODO
}

Client.prototype.dump_config = function(callback) {
	// TODO
}

Client.prototype.stats = function(callback) {
	var self = this;
	self._memcached.stats(callback);
}